local fallingShape   = { l=15, r=25, b=0, t=40 }
local fallingSharp   = { l=10, r=30, b=0, t=20 }

local function Put(x, y, restingSprite, fallingImage, height, sound, stage)
	height = height or 512
	local function PlayFallingSound()
		if sound then 
			eapi.PlaySound(gameWorld, sound)
		end
	end
	local function FallingFall(actor)
		local function EndFalling()
			local size = {50, 116}
			local expl = { "mslug/explosion.png", filter = true }
			local explAnim = eapi.TextureToSpriteList(expl, size)
			eapi.Destroy(actor.tile)
			actor.tile = eapi.NewTile(actor.body, {-40, -10},
						  {100, 232}, explAnim, 0)
			eapi.Animate(actor.tile, eapi.ANIM_CLAMP, 32)

			PlayFallingSound()

			eapi.pointerMap[actor.killZoneObj] = nil
			eapi.Destroy(actor.killZoneObj)

			eapi.AddTimer(actor.body, 0.85, actor.Delete)	
			actor.gravity.x = 0
			actor.vel.x = 0
		end
		actor.HitGround = EndFalling
		object.SimpleStep(actor)		
	end

	local fallingTrigger = {l = -64, r = 64, b = -height, t = 0}

	local drift = 0
	
	if stage == 2 then
		drift = -200 + 400 * util.Random()
	end

	local brownianFN = object.ProduceBrownian(util.Random(0, 50))

	local falling = {    x			= x,
			     y			= y,
			     w			= 40,
			     h			= 80,
			     health		= 1,
			     depth		= 0.5,
			     pressure		= 1000,
			     motionModifier	= brownianFN,
			     gravity		= {x=drift, y=-500},
			     vel		= {x=0, y=0},
			     shape		= fallingShape,
			     killZone		= fallingSharp,
			     restingSprite	= restingSprite,
			     dieImage		= fallingImage,
			     distance		= fallingTrigger,
			     Activate		= FallingFall,
			     TouchPlayer	= destroyer.FallDeath,
			     vulnerable		= true,
			     OnDeath		= PlayFallingSound,
			     contact		= {}
		     }
	object.CompleteHalt(falling)
	action.MakeActor(falling)
	return falling
end

falling = {
	Put = Put
}
return falling
