dofile("script/object.lua")
dofile("script/occlusion.lua")
dofile("script/Proximity.lua")

local activation = { }

local function RemoveActivatorInfo(activator)
	if activator.info then
		eapi.Destroy(unpack(activator.info))
	end
	activator.info = nil
end

local function PlayerActivate(world, playerShape, activatorShape, resolve)
	local activator = activation[activatorShape]
	local player = eapi.pointerMap[playerShape]

	if not(activator.info) and activator.text then
		activator.info = util.GameInfo(activator.text, camera)
	end
	
	activator.active = true
	player.activator = activator

	if player.contact.ground and player.Up() then
		activator.fn()
	end
end

local function MakeActivator(shape, fn, text, body)
	local activator = { }
	body = body or staticBody
	activator.fn = fn
	activator.text = text
	activator.shape = eapi.NewShape(body, nil, shape, "Activator")
	activation[activator.shape] = activator
	return activator
end

local function DeleteActivator(activator)
	eapi.Destroy(activator.shape)
	activation[activator.shape] = nil
end

local function MakeMessage(text, box, lines, AfterMsgCallBack)
	local function ShowMessage(Continue)
		local function NewContinue()
			if AfterMsgCallBack then
				AfterMsgCallBack()
			end
			Continue()			
		end
		local text = lines
		if type(lines) == "function" then
			text = lines()
		end
		if type(text[1]) == "string" then
			util.GameMessage(text, camera, NewContinue)
		else
			local i = 1
			repeat
				local fn = not(text[i + 1]) and NewContinue
				util.GameMessage(text[i], camera, fn)
				i = i + 1
			until not(text[i])
		end
		return true
	end
	local function Msg()
		destroyer.Activate(ShowMessage)
	end
	return action.MakeActivator(box, Msg, text)
end

eapi.Collide(gameWorld, "Player", "Activator", PlayerActivate, 50)

local colliders = { }

local function Collider(world, shape1, shape2, resolve)
	local collider = colliders[shape2]
	collider.fn(weapons.GetProjectile(shape1))
end

local function MakeCollider(shape, fn, body)
      local collider = { }
      collider.fn = fn      
      body = body or staticBody
      if (type(shape) == "table") then
	      collider.shapeObj = eapi.NewShape(body, nil, shape, "Actor")
      else
	      collider.shapeObj = shape
      end
      colliders[collider.shapeObj] = collider
      return collider
end

local function KillPlayer(world, playerShape, actorShape, resolve)
	local player = eapi.pointerMap[playerShape]
	local actor = eapi.pointerMap[actorShape]
	if actor and actor.TouchPlayer then
		actor.TouchPlayer(player, actor)
	else
		destroyer.FallDeath(player)
	end
end

local function Touch(world, playerShape, actorShape, resolve)
	local actor = eapi.pointerMap[actorShape]
	if actor then
		if not(actor.isActive) and actor.TouchInactive then
			actor.TouchInactive()
		end
	end
end

local function OnGround(actor)
	return actor.contact and actor.contact.ground
end

--[[
  * actor rests peacefully until it is triggered by proximity of player
  * actor can be shot by player in which case it shatters
  * actor can kill player by touching it
]]--

local function MakeActor(actor)
	actor.isActive = false
	actor.side = actor.side or false
	actor.body = eapi.NewBody(gameWorld, { x = actor.x, y = actor.y })
	actor.tile = eapi.NewTile(actor.body, { x = 0, y = 0 }, nil,
				  actor.restingSprite, actor.depth)
	eapi.SetAttributes(actor.tile, { flip = { actor.side, false } })
	eapi.SetFrame(actor.tile, 0)

	actor.shapeObj = eapi.NewShape(actor.body, nil, actor.shape, "Actor")
	eapi.pointerMap[actor.shapeObj] = actor

	local function RandomScatter()
		return
		{ x = actor.pressure * util.Random() - 0.5 * actor.pressure,
		  y = actor.pressure * util.Random() }
	end

	actor.Delete = function()
		if actor.proximity then
			actor.proximity.Delete()
			actor.proximity = nil
		end
		object.Delete(actor)
	end

	local function Freeze()
		if OnGround(actor) then
			if actor.body then
				local pos = eapi.GetPos(actor.body)
				actor.Delete()
				return actor.Put(pos.x, pos.y)
			end
		else
			eapi.AddTimer(gameWorld, 0.5, Freeze)
		end
	end

	actor.Die = function()
		if actor.OnDeath then actor.OnDeath(actor) end
		local fn = actor.Scatter or RandomScatter
		effects.Shatter(actor.dieImage, actor, fn, actor.duration or 2)
		if actor.useGibs then
			effects.Gibs(actor, fn, actor.gibsDuration)
		end
		actor.Delete()
	end

	actor.WakeUp = function()
		actor.killZoneObj = eapi.NewShape(actor.body, nil,
						  actor.killZone,
						  "KillerActor")
		eapi.pointerMap[actor.killZoneObj] = actor
		actor.Activate(actor)
	end

	local function ShouldWake()
		return not(actor.WakeCondition) or actor.WakeCondition()
	end

	local function MaybeActivate()
		if ShouldWake() and not(actor.isActive) then
			eapi.AddTimer(actor.body, 0, actor.WakeUp)
			actor.isActive = true
		end
	end

	if actor.distance then
		local box = nil
		if type(actor.distance) == "number" then
			box = { l = -actor.distance, r = actor.distance,
				b = -actor.distance, t = actor.distance }
		else		
			box = { l = actor.distance.l, r = actor.distance.r,
				b = actor.distance.b, t = actor.distance.t }
		end
		box.r = box.r + actor.w
		box.t = box.t + actor.h

		local Deactivate = (actor.freeze and Freeze) or function() end
		actor.proximity = proximity.Create(MaybeActivate, Deactivate,
						   nil, box, actor.body)
	end

	local function Shoot(projectile)
		if actor.isActive or actor.vulnerable then
			MaybeActivate()
			if actor.ShootEffect then
				actor.ShootEffect(actor, projectile)
			end
			actor.health = actor.health - projectile.damage
			weapons.DeleteProjectile(projectile)
			if actor.health <= 0 then
				local Die = actor.AlternateDie or actor.Die
				Die(actor)
			end
		end
	end

	action.MakeCollider(actor.shapeObj, Shoot, actor.body)
end

eapi.Collide(gameWorld, "Player", "KillerActor", KillPlayer, -100)
eapi.Collide(gameWorld, "Player", "Actor", Touch, -100)
eapi.Collide(gameWorld, "Projectile", "Actor", Collider, -100)

object.RegisterHandlers("Actor")

local allWaypoints = { }

local function WayPointHandler(world, waypointShape, actorShape, resolve)
	local actor = eapi.pointerMap[actorShape]
	if actor.WayPointHandler then
		actor.WayPointHandler(actor, allWaypoints[waypointShape])
	end
end

eapi.Collide(gameWorld, "WayPoint", "Actor", WayPointHandler, -100)

local function CreateWaypoint(shape, action)
	local shapeObj = eapi.NewShape(staticBody, nil, shape, "WayPoint")
	allWaypoints[shapeObj] = action
end

local function WaypointFunction(action)
	return function(pos)
		CreateWaypoint({ l = pos.x, r = pos.x + 2,
				 b = pos.y, t = pos.y + 2 },
			       action)
	end
end

local function JumpBackWaypoint(pos)
	CreateWaypointFromPosition(pos, "jump-back")
end

local function LineByLine(lines)
	local collection = { }
	for i = 1, #lines do
		collection[i] = { }
		for j = 1, i do
			collection[i][j] = lines[j]
		end
	end
	return collection
end

local function DistanceToPlayer(actor)
	local body = actor.body
	return eapi.GetPos(mainPC.body).x - eapi.GetPos(body).x - actor.w / 2
end

local function HeightDifference(body)
	return eapi.GetPos(mainPC.body).y - eapi.GetPos(body).y
end

local function ShouldTurn(distance, side)
	return (distance > 0 and side) or (distance < 0 and not(side))
end

local function GetKickDirection(actor)
	if mainPC.dead then
		if not(actor.randomDirection) then
			local coin = (util.Random() > 0.5)
			actor.randomDirection = (coin and 1) or -1
		end
		return actor.randomDirection
	end
	local delta = DistanceToPlayer(actor)
	if delta == 0 then
		return 0
	else
		return delta / math.abs(delta)
	end
end

local function SetDirection(actor)
	local tile = actor.tile
	local attribute = { flip = { not(actor.side), false } }
	eapi.SetAttributes(tile, attribute)
end

local function Crush(volume)
	return function()
		eapi.PlaySound(gameWorld, "sound/crush.ogg", 0, volume)
	end
end

action = {
	MakeActivator = MakeActivator,
	MakeCollider  = MakeCollider,
	MakeActor     = MakeActor,
	MakeMessage   = MakeMessage,
	LineByLine    = LineByLine,
	DeleteActivator = DeleteActivator,
	RemoveActivatorInfo = RemoveActivatorInfo,
	DistanceToPlayer = DistanceToPlayer,
	HeightDifference = HeightDifference,
	GetKickDirection = GetKickDirection,
	ShouldTurn = ShouldTurn,
	SetDirection = SetDirection,
	OnGround = OnGround,
	WaypointFunction = WaypointFunction,
	Crush = Crush,
}
return action

