dofile("script/action.lua")
dofile("script/occlusion.lua")
dofile("script/Falling.lua")

local info = { bombs = {}, speed = {} }

local function ResetCounters()
	info.bombs[1] = 50
	info.speed[1] = 2.0
	
	info.bombs[2] = 50
	info.speed[2] = 1.0
	
	info.bombs[3] = 15
	info.speed[3] = 4.0
end

ResetCounters()

LoadPlayers()
camera = util.CreateCamera(gameWorld)
eapi.SetBackgroundColor(gameWorld, {r=0.0, g=0.0, b=0.0})

staticBody = eapi.GetStaticBody(gameWorld)

local tx = eapi.NewSpriteList("mslug/background.png", {{0, 0}, {800, 480}})
eapi.NewTile(staticBody, {x=-400,y=-240}, nil, tx, -200.0)

eapi.NewShape(staticBody, nil, {l=-400, r=400, b=-240, t=-200}, "Box")
eapi.NewShape(staticBody, nil, {l=-500, r=-400, b=-240, t=240}, "Box")
eapi.NewShape(staticBody, nil, {l=400, r=500, b=-240, t=240}, "Box")

local expl = "mslug/explode.ogg"

local missile = eapi.NewSpriteList("mslug/missile.png", {{0, 0}, {40, 80}})

local missileImage = {
	image		= "mslug/missile.png",
	spriteOffset	= { x = 0, y = 0 },
}

local LastFn = nil

local function Pause(time, fn)
	local function Check()
		if not(mainPC.dead) then
			LastFn = fn
			fn()
		else
			ResetCounters()
			eapi.Destroy(mainPC.body)
			mainPC = destroyer.Create()
			destroyer.Load(gameWorld, mainPC)
			eapi.SetPos(mainPC.body, {x = 0, y = 300})
			LastFn()
		end
	end
	eapi.AddTimer(staticBody, time, Check)		
end

local textSize = {{0, 0}, {600, 45}}
local function Advertise(stageFile, soundFile, nameFile, followUp)
	return function()
		local name = eapi.NewSpriteList(nameFile, textSize)
		local stage = eapi.NewSpriteList(stageFile, textSize)
		local tile1 = eapi.NewTile(staticBody, {-1150,100},
					   {1200, 90}, stage, 10)
		local tile2 = eapi.NewTile(staticBody, {-300, -250},
					   {600, 45}, name, 10)
		local function Remove()
			local moved = false
			local pos = eapi.GetPos(tile1)
			if pos.x < 600 then
				pos.x = pos.x + 5
				eapi.SetPos(tile1, pos)
				moved = true
			end
			local pos = eapi.GetPos(tile2)
			if pos.x < 400 then
				pos.x = pos.x + 5
				eapi.SetPos(tile2, pos)
				moved = true
			end
			if moved then
				eapi.AddTimer(staticBody, 0.01, Remove)
			else
				eapi.Destroy(tile1)
				eapi.Destroy(tile2)
				followUp()
			end
		end
		local function Move()
			local moved = false
			local pos = eapi.GetPos(tile1)
			if pos.x < -600 then
				pos.x = pos.x + 5
				eapi.SetPos(tile1, pos)
				moved = true
			end
			local pos = eapi.GetPos(tile2)
			if pos.y < 80 then
				pos.y = pos.y + 3
				eapi.SetPos(tile2, pos)
				moved = true
			end
			if moved then
				eapi.AddTimer(staticBody, 0.01, Move)
			else
				eapi.AddTimer(staticBody, 1.0, Remove)
				eapi.PlaySound(gameWorld, soundFile, 0, 1.0)
			end
		end
		Move()
		
	end
end

local function Bomb(x, stage)
	local item = falling.Put(x, 300, missile, missileImage, 0, expl, stage)
	eapi.PlaySound(gameWorld, "mslug/bomb.ogg", 0, 1.0)
	item.isActive = true
	item.WakeUp()
end

local function Hole()
	return (util.Random(0, 4) - 2) * 80
end

local function BombRow(dir)
	local x = -dir * 400
	local skip = Hole()
	local function Next()
		if not(x == skip) then Bomb(x, 3) end
		x = x + dir * 80
		if math.abs(x) < 401  then
			eapi.AddTimer(staticBody, 0.2, Next)
		end
	end
	Next()
end

local function ManyBombs()
	BombRow(1)
end

local function SingleBomb(stage)
	local x = -400 + 800 * util.Random()
	Bomb(x, stage)
end

local function Stage1or2or3(stage, fn, accel, action, next)
	if info.bombs[stage] > 1 and not(mainPC.dead) then
		eapi.AddTimer(staticBody, info.speed[stage], fn)
		info.bombs[stage] = info.bombs[stage] - 1
	else
		Pause(5.0, next)
	end
	local minSpace = (stage == 3) and 2.0 or 0.5
	info.speed[stage] = math.max(minSpace, info.speed[stage] - accel)
	action(stage)
end

local function CalcCakePos()
	local pos = eapi.GetPos(mainPC.body)	
	local x = pos.x + ((mainPC.direction and -100) or 50)
	if math.abs(x) > 380 then
		return 0
	else
		return x
	end
end

local function DropCake()
	local hp = { }
	local pos = { x = CalcCakePos(), y = 450}
	local cakeFile = { "mslug/cake.png", filter = true }
	local img = eapi.NewSpriteList(cakeFile, {{0,0},{20,19}})
	local shape = { b = 5, t = 15, l = -10, r = 10 } 
	hp.body = eapi.NewBody(gameWorld, pos)
	hp.tile = eapi.NewTile(hp.body, zero, {40,40}, img, -0.2)
	hp.shape = eapi.NewShape(hp.body, nil, shape, "Figurine")
	hp.vel = { x = 0, y = 0 }
	hp.gravity = { x = 0, y = -500 }
	eapi.pointerMap[hp.shape] = hp
	object.RegisterHandlers("Figurine")
	object.CompleteHalt(hp)
	object.SimpleStep(hp)
end

local function End()
	local player = mainPC
	player.StopInput()
	player.stopped = true
	local cheer = { "mslug/cheer.png", filter = true }
	local cheerAnim = eapi.TextureToSpriteList(cheer, {33, 47})
	eapi.Destroy(player.tile)
	player.tile = eapi.NewTile(player.body, {-45,-20},
				   {90, 128}, cheerAnim, 0)
	local attrib = {flip = {not(player.direction), false}}
	eapi.SetAttributes(player.tile, attrib)
	eapi.Animate(player.tile, eapi.ANIM_LOOP, 8)

	local name = eapi.NewSpriteList("mslug/you-are-great.png", textSize)
	local tile = eapi.NewTile(staticBody, {-600, 400},
				   {1200, 90}, name, 10)
	local function Move()
		local moved = false
		local pos = eapi.GetPos(tile)
		if pos.y > 150 then
			pos.y = pos.y - 2
			eapi.SetPos(tile, pos)
			moved = true
		end
		if moved then			
			eapi.AddTimer(staticBody, 0.01, Move)
		else
			local fileName = "mslug/you-are-great.ogg"
			eapi.PlaySound(gameWorld, fileName, 0, 1.0)
			DropCake()
		end
	end
	Move()
end

local function Stage3()
	Stage1or2or3(3, Stage3, 0.20, ManyBombs, End)
end

local function Stage2()
	Stage1or2or3(2, Stage2, 0.05, SingleBomb,
		     Advertise("mslug/stage3.png",
			       "mslug/stage3.ogg",
			       "mslug/carpet-bombing.png",
			       Stage3))
end

local function Stage1()
	Stage1or2or3(1, Stage1, 0.10, SingleBomb,
		     Advertise("mslug/stage2.png",
			       "mslug/stage2.ogg",
			       "mslug/bombs-in-the-wind.png",
			       Stage2))
end

local function Start()
	Pause(2.0, Advertise("mslug/stage1.png",
			     "mslug/stage1.ogg",
			     "mslug/heavy-rain.png",
			     Stage1))
end

local attr = { size = { 800, 480 } }
local tile = Occlusion.put('f', -400, -240, 90, attr, camera.ptr)

local function FadeIn()
	eapi.PlaySound(gameWorld, "mslug/back-to-the-china.ogg", -1, 0.5)
	effects.Fade(1.0, 0.0, 2.0, Start, tile)
end

util.GameMessage({ "MISERABLE SLUG",
		   "--------------",
		   "", "Z - start game"}, camera, FadeIn)

eapi.BindKey(eapi.KEY_ESCAPE, function() eapi.Quit() end)